"use strict";

var _path = _interopRequireDefault(require("path"));

var _codeFrame = require("@babel/code-frame");

var _typescript = _interopRequireDefault(require("typescript"));

var _fs = _interopRequireDefault(require("fs"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var appendCodeFrame = function appendCodeFrame(_ref) {
  var filePath = _ref.filePath,
      errorMessage = _ref.errorMessage,
      location = _ref.location;

  if (typeof location === 'undefined') {
    return errorMessage;
  }

  var rawLines = _fs.default.readFileSync(filePath, 'utf8');

  return "".concat(errorMessage, "\n").concat((0, _codeFrame.codeFrameColumns)(rawLines, location, {
    highlightCode: true
  }));
};

var convertErrors = function convertErrors(_ref2) {
  var start = _ref2.start,
      end = _ref2.end,
      errors = _ref2.errors,
      testPath = _ref2.testPath;
  return {
    console: null,
    failureMessage: errors.map(appendCodeFrame).join('\n\n'),
    numFailingTests: errors.length,
    numPassingTests: errors.length ? 0 : 1,
    numPendingTests: 0,
    perfStats: {
      end: end,
      start: start
    },
    snapshot: {
      added: 0,
      fileDeleted: false,
      matched: 0,
      unchecked: 0,
      unmatched: 0,
      updated: 0
    },
    sourceMaps: {},
    testExecError: null,
    testFilePath: testPath,
    testResults: (errors.length ? errors : [{
      filePath: testPath
    }]).map(function (error) {
      return {
        ancestorTitles: [],
        duration: end - start,
        failureMessages: [error.errorMessage],
        fullName: error.filePath,
        numPassingAsserts: error.errorMessage ? 1 : 0,
        status: error.errorMessage ? 'failed' : 'passed',
        title: 'tsc'
      };
    })
  };
};

var runTsc = function runTsc(_ref3, workerCallback) {
  var testPath = _ref3.testPath,
      jestConfig = _ref3.config;

  try {
    var configPath = _path.default.resolve(jestConfig.rootDir, 'tsconfig.json');

    var configContents = _fs.default.readFileSync(configPath).toString();

    var _ts$parseConfigFileTe = _typescript.default.parseConfigFileTextToJson(configPath, configContents),
        config = _ts$parseConfigFileTe.config,
        error = _ts$parseConfigFileTe.error;

    if (error) {
      return {
        errorMessage: error,
        filePath: testPath
      };
    }

    var settings = _typescript.default.convertCompilerOptionsFromJson(config['compilerOptions'] || {}, process.cwd());

    var options = Object.assign({}, {
      noEmit: true
    }, settings.options);
    var start = Date.now();

    var program = _typescript.default.createProgram([testPath], options);

    var emitResult = program.emit();

    var allDiagnostics = _typescript.default.getPreEmitDiagnostics(program).concat(emitResult.diagnostics);

    var errors = allDiagnostics.map(function (diagnostic) {
      if (diagnostic.file) {
        var _diagnostic$file$getL = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start),
            lineStart = _diagnostic$file$getL.line,
            characterStart = _diagnostic$file$getL.character;

        var _diagnostic$file$getL2 = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start + diagnostic.length),
            lineEnd = _diagnostic$file$getL2.line,
            characterEnd = _diagnostic$file$getL2.character;

        var location = {
          start: {
            line: lineStart + 1,
            column: characterStart + 1
          },
          end: {
            line: lineEnd + 1,
            column: characterEnd + 1
          }
        };

        var message = _typescript.default.flattenDiagnosticMessageText(diagnostic.messageText, '\n');

        return {
          location: location,
          errorMessage: message,
          filePath: diagnostic.file.fileName
        };
      } else {
        return {
          errorMessage: "".concat(_typescript.default.flattenDiagnosticMessageText(diagnostic.messageText, '\n')),
          filePath: testPath
        };
      }
    });
    var end = Date.now();
    workerCallback(null, convertErrors({
      start: start,
      end: end,
      errors: errors,
      testPath: testPath
    }));
  } catch (e) {
    workerCallback(e);
  }
};

module.exports = runTsc;